using System;
using System.IO;
using System.IO.Ports;

public delegate void StringEventHandler(object sender, StringEventArgs e);
public class StringEventArgs : EventArgs {
        public string StringValue;
        public StringEventArgs(string s){
                StringValue = s;
        }
}

static class Program
{

    static Stream fstream;

    static void printMessage(object sender, StringEventArgs e)
    {
        Console.WriteLine("{0}", e.StringValue);
    }
    
	static int Main(string[] args)
	{
        string[] ports = SerialPort.GetPortNames();
        Console.WriteLine("The following serial ports were found:");
        foreach(string port in ports)
            Console.Write("{0} ", port);
        Console.WriteLine("");

        if(args.Length != 3){
            Console.WriteLine("Error: must supply 3 arguments");
            Console.WriteLine("Usage: $ bootfirm comport baudrate filepath");
            Console.WriteLine("baudrate must be one of 9600, 19200, 38400, 57600, 115200");
            return -1;
        }
        string commPort = args[0];

        int baudRate = 9600;
        try{
            baudRate = Int32.Parse(args[1]);
            if(baudRate != 9600 && baudRate != 19200 && baudRate != 38400 &&
               baudRate != 57600 && baudRate != 115200){
                throw new Exception();
            }
        } catch{
            Console.WriteLine("Error: baud rate must be one of 9600, 19200, 38400, 57600, 115200");
            return -1;
        }
        
        
        string imageFile = args[2];

        Console.WriteLine("firmware image: {0}", imageFile);
        try{
            fstream = new FileStream(imageFile, FileMode.Open);
        } catch (Exception E){     
            Console.WriteLine("Error: {0}", E.Message);
            return -1;
        }

        
        FirmwareDownloader down = new FirmwareDownloader(commPort, fstream, baudRate);
        down.EmitMessage += new StringEventHandler(printMessage);
        down.Download();


        return 0;
	}
}
